﻿using System;
using System.Collections.Generic;
using System.IO;

namespace bankProductAdvertising.Class
{
	public class ComplaintSystem
	{
		private Dictionary<String, Customer> customerDetails;
		private Dictionary<String, BankSalesPerson> bankSalesPersonDetails;
		private List<Complaint> complaints;
		private MessageHandler messageHandler;

		public ComplaintSystem(UploadDetails details, MessageHandler messageHandler)
		{
			this.bankSalesPersonDetails = details.BankSalesPersonDetail;
			this.customerDetails = details.CustomerDetail;
			this.messageHandler = messageHandler;
			this.complaints = new List<Complaint>();
			this.loadComplaints();
		}

		public ComplaintSystem()
		{
		}

		private void loadComplaints()
		{
			using (StreamReader reader = new StreamReader(@"../../Resources/CustomerComplaints.csv")) {
				while (!reader.EndOfStream) {
					String line = reader.ReadLine();
					String[] values = line.Split('|');
					Complaint complaint = new Complaint(values[0], values[1], values[2]);
					this.complaints.Add(complaint);
				}
			}
		}

		public void updateComplaints()
		{
			foreach (Complaint complaint in this.complaints) {
				Customer customer;
				BankSalesPerson bankSalesPerson = this.getBankPerson(complaint.Type);
				String message;
				this.customerDetails.TryGetValue(complaint.CustomerId, out customer);
				if (complaint.Type.Equals("COMPLAINT")) {
					message = "Hi Mr/Ms" + customer.getFullName() + ", on your complaint: " + complaint.Content + ", Mr/Ms " + bankSalesPerson.getFullName() + " will contact you with phone number: " + bankSalesPerson.PhoneNumber;
				} else if (complaint.Type.Equals("QUERY")) {
					message = "Hi Mr/Ms" + customer.getFullName() + ", on your query: " + complaint.Content + ". Please connect to our sales person Mr/Ms " + bankSalesPerson.getFullName() + " on " + bankSalesPerson.PhoneNumber + " or our sales person will conect with you in the next 3 working days.";
				} else if (complaint.Type.Equals("CRITICAL")) {
					message = "Hi Mr/Ms" + customer.getFullName() + ", on your issue: " + complaint.Content + ", an appointment with our manager is being fixed on " + this.getDateAndTime() + ".";
				} else {
					message = "";
				}
				if (!message.Equals("")) {
					messageHandler.sendMessage(message, customer.PhoneNumber);
				}
			}
		}

		private BankSalesPerson getBankPerson(String complaintType)
		{
			BankSalesPerson salesPerson;
			if (complaintType.Equals("COMPLAINT")) {
				bankSalesPersonDetails.TryGetValue("3", out salesPerson);
			} else if (complaintType.Equals("QUERY")) {
				bankSalesPersonDetails.TryGetValue("2", out salesPerson);
			} else if (complaintType.Equals("CRITICAL")) {
				bankSalesPersonDetails.TryGetValue("1", out salesPerson);
			} else {
				salesPerson = null;
			}
			return salesPerson;
		}

		private String getDateAndTime()
		{
			DateTime sysDate = DateTime.Now;
			Int32 days = (sysDate.DayOfWeek == DayOfWeek.Sunday || sysDate.DayOfWeek == DayOfWeek.Saturday) ? 3 : 1;
			DateTime appointment = DateTime.Now.AddDays(days);
			return appointment.ToString("D");
		}
	}
}
